#include <list>
#include <string>

using namespace std;

/* Symbol represents an identifier in program's source code such as a variable
   or function name */
class Symbol {
	string name;
public:
	Symbol() {};
	Symbol(const string &initName) : name(initName) {};

	// used to access the name of the symbol
	string get_name() const { return name; };   
	void set_name(const string &newName) { name = newName; };
};

typedef void (*VisitFunction)(Symbol &, void *);

class SymbolTable {
	list<Symbol> *buckets;
	int tableSize;

	// n % tableSize where n is generated using horner's rule
	int hash(const string &);

public:
	// creates an empty table with n buckets
	SymbolTable(int n);

	// insert a copy of a symbol into the table, will reject duplicates
	void insert(const Symbol &);

	// returns a pointer to the matching symbol, NULL if not found
	Symbol *retrieve(const string &);

	// erases the given symbol, does nothing if symbol is not found
	void erase(const string &);
	
	// executes func(s, arg) for every symbol as s
	void traverse(VisitFunction func, void *arg);

	// clears the entire table
	void clear();

	// changes the number of buckets
	void resize(unsigned int newSize);	
};

