#include "symboltable.h"

SymbolTable::SymbolTable(int initSize) : tableSize(initSize) {
   items = new Symbol[initSize];
}

int SymbolTable::hash(const string &str) {
	int bucketID = str[0];
	for(int i = 1; i < str.size(); i++) {
		bucketID *= 32;
		bucketID += str[i];
		bucketID %= tableSize;
	}
	return bucketID;
}

void SymbolTable::insert(const Symbol &symbol) {
	int pos = hash(symbol.get_name());

	for(int i = pos; i != pos - 1; i++) {
		if(i == tableSize) i = 0;
		if(items[i].state == Empty) {
			items[i] = symbol;
			items[i].state = Full;
			return;
		} else if(items[i] == symbol) {
			return;
		}
	}
}

Symbol *SymbolTable::retrieve(const string &name) {
	int pos = hash(name);

	for(int i = pos; i != pos - 1; i++) {
		if(i == tableSize) i = 0;
			if(items[i].state == Empty) {
				return NULL;
			} else if(items[i].state == Full && items[i].get_name() == name) {
		return &(items[i]);
		}
	}
	return NULL;
}

void SymbolTable::erase(const string &name) {
	int pos = hash(name);
	for(int i = pos; i != pos - 1; i++) {
		if(i == tableSize) i = 0;
		if(items[i].state == Empty) {
			return;
		} else if(items[i].get_name() == name) {
			items[i].state = Deleted;
			return;
		}
	}
}

void SymbolTable::traverse(VisitFunction func, void *arg) {
	for(int i = 0; i < tableSize; i++) {
		if(items[i].state == Full) func(items[i], arg);
	}
}

void SymbolTable::clear() {
	for(int i = 0; i < tableSize; i++) {
		items[i].state = Empty;
		items[i].set_name("");
	}
}

void SymbolTable::resize(int newSize) {
	list<Symbol> tempHold;
	for(int i = 0; i < tableSize; i++) {
		if(items[i].state == Full) tempHold.push_front(items[i]);
	}

	delete[] items;

	tableSize = newSize;
	if(tableSize % 2 == 0) {
		tableSize++;
	}

	items = new Symbol[tableSize];

	list<Symbol>::iterator i = tempHold.begin();
	for(int n = 0; i != tempHold.end(); i++, n++) {
		if(n == tableSize) return;
		insert(*i);
	}
}

